@@ -10,27 +10,27 @@ import json
 import pythesint as pti
 
 from nansat.vrt import VRT
-from nansat.tools import WrongMapperError
+from nansat.exceptions import WrongMapperError
 
 
 class Mapper(VRT):
     ''' VRT with mapping of WKV for MOD44W produc (MODIS watermask at 250 m)'''
 
-    def __init__(self, fileName, gdalDataset, gdalMetadata, **kwargs):
+    def __init__(self, filename, gdalDataset, gdalMetadata, **kwargs):
         ''' Create VRT '''
 
-        fileBaseName = os.path.basename(fileName)
+        fileBaseName = os.path.basename(filename)
         if not fileBaseName == 'MOD44W.vrt':
             raise WrongMapperError
 
-        metaDict = [{'src': {'SourceFilename': fileName, 'SourceBand':  1},
+        metaDict = [{'src': {'SourceFilename': filename, 'SourceBand':  1},
                      'dst': {'wkv': 'land_binary_mask'}}]
 
         # create empty VRT dataset with geolocation only
-        VRT.__init__(self, gdalDataset)
+        self._init_from_gdal_dataset(gdalDataset)
 
         # add bands with metadata and corresponding values to the empty VRT
-        self._create_bands(metaDict)
+        self.create_bands(metaDict)
 
         mm = pti.get_gcmd_instrument('MODIS')
         ee = pti.get_gcmd_platform('TERRA')
