@@ -1,6 +1,6 @@
 from nansat.mappers.mapper_arome import Mapper as MapperArome
 from nansat.mappers.opendap import Opendap
-from nansat.tools import WrongMapperError
+from nansat.exceptions import WrongMapperError
 
 class Mapper(Opendap, MapperArome):
 
@@ -8,7 +8,7 @@ class Mapper(Opendap, MapperArome):
 
     def __init__(self, *args, **kwargs):
 
-        raise WrongMapperError
+        raise WrongMapperError('Mapper is under development...')
 
         fn = args[0]
         #ds = args[1] - None
